// Copyright 2018 JDCLOUD.COM
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// NOTE: This class is auto generated by the jdcloud code generator program.

package models


type InstanceType struct {

    /* 实例规格类型 (Optional) */
    Family string `json:"family"`

    /* 实例规格，比如g.b1.2xlarge (Optional) */
    InstanceType string `json:"instanceType"`

    /* cpu个数 (Optional) */
    Cpu int `json:"cpu"`

    /* 内存大小 (Optional) */
    MemoryMB int `json:"memoryMB"`

    /* 支持弹性网卡的数量 (Optional) */
    NicLimit int `json:"nicLimit"`

    /* 描述 (Optional) */
    Desc string `json:"desc"`

    /* 规格状态 (Optional) */
    State []InstanceTypeState `json:"state"`

    /* Gpu配置 (Optional) */
    Gpu Gpu `json:"gpu"`

    /* 本地缓存盘配置，目前只有Gpu规格上才有 (Optional) */
    LocalDisks []LocalDisk `json:"localDisks"`
}
