/*
 * server
 *
 * <br/>https://ncloud.apigw.ntruss.com/server/v2
 *
 * API version: 2018-10-18T06:16:13Z
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package server

type GetServerInstanceListRequest struct {

	// 서버인스턴스번호리스트
ServerInstanceNoList []*string `json:"serverInstanceNoList,omitempty"`

	// 검색필터명
SearchFilterName *string `json:"searchFilterName,omitempty"`

	// 검색필터값
SearchFilterValue *string `json:"searchFilterValue,omitempty"`

	// 페이지번호
PageNo *int32 `json:"pageNo,omitempty"`

	// 페이지사이즈
PageSize *int32 `json:"pageSize,omitempty"`

	// 서버인스턴스상태코드
ServerInstanceStatusCode *string `json:"serverInstanceStatusCode,omitempty"`

	// 인터넷라인구분코드
InternetLineTypeCode *string `json:"internetLineTypeCode,omitempty"`

	// 리전번호
RegionNo *string `json:"regionNo,omitempty"`

	// zone번호
ZoneNo *string `json:"zoneNo,omitempty"`

	// 서버디스크유형코드
BaseBlockStorageDiskTypeCode *string `json:"baseBlockStorageDiskTypeCode,omitempty"`

	// 서버디스크상세유형코드
BaseBlockStorageDiskDetailTypeCode *string `json:"baseBlockStorageDiskDetailTypeCode,omitempty"`

	// 소팅대상
SortedBy *string `json:"sortedBy,omitempty"`

	// 소팅순서
SortingOrder *string `json:"sortingOrder,omitempty"`

	// 서비인스턴스구분코드리스트
ServerInstanceTypeCodeList []*string `json:"serverInstanceTypeCodeList,omitempty"`

	// 태그키리스트
TagKeyList []*string `json:"tagKeyList,omitempty"`

	// 태그값리스트
TagValueList []*string `json:"tagValueList,omitempty"`
}
