// Code generated by "stringer -type JobStatusType"; DO NOT EDIT.

package egoscale

import "strconv"

const _JobStatusType_name = "PendingSuccessFailure"

var _JobStatusType_index = [...]uint8{0, 7, 14, 21}

func (i JobStatusType) String() string {
	if i < 0 || i >= JobStatusType(len(_JobStatusType_index)-1) {
		return "JobStatusType(" + strconv.FormatInt(int64(i), 10) + ")"
	}
	return _JobStatusType_name[_JobStatusType_index[i]:_JobStatusType_index[i+1]]
}
