// Copyright 2018 JDCLOUD.COM
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// NOTE: This class is auto generated by the jdcloud code generator program.

package models


type InstanceTemplateData struct {

    /* 实例规格 (Optional) */
    InstanceType string `json:"instanceType"`

    /* 主网卡所属VPC的ID (Optional) */
    VpcId string `json:"vpcId"`

    /* 镜像ID (Optional) */
    ImageId string `json:"imageId"`

    /* 启动模板中是否包含自定义密码，true：包含密码，false：不包含密码 (Optional) */
    IncludePassword bool `json:"includePassword"`

    /* 系统盘信息 (Optional) */
    SystemDisk InstanceTemplateDiskAttachment `json:"systemDisk"`

    /* 数据盘信息，本地盘(local类型)做系统盘的云主机可挂载8块数据盘，云硬盘(cloud类型)做系统盘的云主机可挂载7块数据盘。 (Optional) */
    DataDisks []InstanceTemplateDiskAttachment `json:"dataDisks"`

    /* 主网卡信息 (Optional) */
    PrimaryNetworkInterface InstanceTemplateNetworkInterfaceAttachmentSpec `json:"primaryNetworkInterface"`

    /* 主网卡主IP关联的弹性IP规格 (Optional) */
    ElasticIp InstanceTemplateElasticIp `json:"elasticIp"`

    /* 密钥对名称；当前只支持一个 (Optional) */
    KeyNames []string `json:"keyNames"`
}
