package ram

//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
// Code generated by Alibaba Cloud SDK Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

// User is a nested struct in ram response
type User struct {
	MobilePhone   string `json:"MobilePhone" xml:"MobilePhone"`
	Comments      string `json:"Comments" xml:"Comments"`
	CreateDate    string `json:"CreateDate" xml:"CreateDate"`
	AttachDate    string `json:"AttachDate" xml:"AttachDate"`
	Email         string `json:"Email" xml:"Email"`
	UserId        string `json:"UserId" xml:"UserId"`
	UpdateDate    string `json:"UpdateDate" xml:"UpdateDate"`
	UserName      string `json:"UserName" xml:"UserName"`
	JoinDate      string `json:"JoinDate" xml:"JoinDate"`
	LastLoginDate string `json:"LastLoginDate" xml:"LastLoginDate"`
	DisplayName   string `json:"DisplayName" xml:"DisplayName"`
}
