//<!-- Code generated by generate-fixer-deprecations; DO NOT EDIT MANUALLY -->

package config

var DeprecatedOptions = map[string][]string{
	"*":                                   []string{"iso_md5", "ssh_key_path", "ssh_disable_agent", "ssh_host_port_min", "ssh_host_port_max", "ssh_skip_nat_mapping", "ssh_wait_timeout", "iso_checksum_url", "iso_checksum_type"},
	"*amazon*":                            []string{"shutdown_behaviour", "enhanced_networking", "ssh_private_ip", "temporary_security_group_source_cidr", "clean_ami_name", "spot_price_auto_product"},
	"Azure*":                              []string{"clean_image_name", "exlude_from_latest"},
	"MSOpenTech.hyperv":                   []string{"vhd_temp_path", "clone_from_vmxc_path", "cpu", "ram_size"},
	"ansible":                             []string{"galaxycommand"},
	"hashicorp.scaleway":                  []string{"access_key"},
	"jetbrains.vsphere":                   []string{"network_card", "network", "networkCard", "disk_size", "disk_thin_provisioned", "disk_eagerly_scrub"},
	"mitchellh.virtualbox":                []string{"guest_additions_attach"},
	"packer.docker":                       []string{"login_email"},
	"packer.googlecompute":                []string{"clean_image_name"},
	"packer.parallels":                    []string{"headless", "parallels_tools_host_path", "guest_os_distribution"},
	"packer.post-processor.docker-import": []string{"login_email"},
	"packer.post-processor.docker-tag":    []string{"tag"},
	"packer.post-processor.manifest":      []string{"filename"},
	"transcend.qemu":                      []string{"ssh_host_port_max", "ssh_host_port_min"},
}
