package ecs

//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
// Code generated by Alibaba Cloud SDK Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

// LaunchTemplateSet is a nested struct in ecs response
type LaunchTemplateSet struct {
	CreateTime           string                        `json:"CreateTime" xml:"CreateTime"`
	ModifiedTime         string                        `json:"ModifiedTime" xml:"ModifiedTime"`
	LaunchTemplateId     string                        `json:"LaunchTemplateId" xml:"LaunchTemplateId"`
	LaunchTemplateName   string                        `json:"LaunchTemplateName" xml:"LaunchTemplateName"`
	DefaultVersionNumber int                           `json:"DefaultVersionNumber" xml:"DefaultVersionNumber"`
	LatestVersionNumber  int                           `json:"LatestVersionNumber" xml:"LatestVersionNumber"`
	CreatedBy            string                        `json:"CreatedBy" xml:"CreatedBy"`
	ResourceGroupId      string                        `json:"ResourceGroupId" xml:"ResourceGroupId"`
	Tags                 TagsInDescribeLaunchTemplates `json:"Tags" xml:"Tags"`
}
