// Code generated by "mapstructure-to-hcl2 -type RunConfig"; DO NOT EDIT.
package common

import (
	"github.com/hashicorp/hcl/v2/hcldec"
	"github.com/zclconf/go-cty/cty"
)

// FlatRunConfig is an auto-generated flat version of RunConfig.
// Where the contents of a field with a `mapstructure:,squash` tag are bubbled up.
type FlatRunConfig struct {
	BootOrder *string `mapstructure:"boot_order" cty:"boot_order" hcl:"boot_order"`
}

// FlatMapstructure returns a new FlatRunConfig.
// FlatRunConfig is an auto-generated flat version of RunConfig.
// Where the contents a fields with a `mapstructure:,squash` tag are bubbled up.
func (*RunConfig) FlatMapstructure() interface{ HCL2Spec() map[string]hcldec.Spec } {
	return new(FlatRunConfig)
}

// HCL2Spec returns the hcl spec of a RunConfig.
// This spec is used by HCL to read the fields of RunConfig.
// The decoded values from this spec will then be applied to a FlatRunConfig.
func (*FlatRunConfig) HCL2Spec() map[string]hcldec.Spec {
	s := map[string]hcldec.Spec{
		"boot_order": &hcldec.AttrSpec{Name: "boot_order", Type: cty.String, Required: false},
	}
	return s
}
