// Code generated by "enumer -type Algorithm -transform snake"; DO NOT EDIT.

//
package sshkey

import (
	"fmt"
)

const _AlgorithmName = "rsadsaecdsaed25519"

var _AlgorithmIndex = [...]uint8{0, 3, 6, 11, 18}

func (i Algorithm) String() string {
	if i < 0 || i >= Algorithm(len(_AlgorithmIndex)-1) {
		return fmt.Sprintf("Algorithm(%d)", i)
	}
	return _AlgorithmName[_AlgorithmIndex[i]:_AlgorithmIndex[i+1]]
}

var _AlgorithmValues = []Algorithm{0, 1, 2, 3}

var _AlgorithmNameToValueMap = map[string]Algorithm{
	_AlgorithmName[0:3]:   0,
	_AlgorithmName[3:6]:   1,
	_AlgorithmName[6:11]:  2,
	_AlgorithmName[11:18]: 3,
}

// AlgorithmString retrieves an enum value from the enum constants string name.
// Throws an error if the param is not part of the enum.
func AlgorithmString(s string) (Algorithm, error) {
	if val, ok := _AlgorithmNameToValueMap[s]; ok {
		return val, nil
	}
	return 0, fmt.Errorf("%s does not belong to Algorithm values", s)
}

// AlgorithmValues returns all values of the enum
func AlgorithmValues() []Algorithm {
	return _AlgorithmValues
}

// IsAAlgorithm returns "true" if the value is listed in the enum definition. "false" otherwise
func (i Algorithm) IsAAlgorithm() bool {
	for _, v := range _AlgorithmValues {
		if i == v {
			return true
		}
	}
	return false
}
