// Copyright 2018 JDCLOUD.COM
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// NOTE: This class is auto generated by the jdcloud code generator program.

package models


type Segment struct {

    /* 网段的Id (Optional) */
    SegmentId string `json:"segmentId"`

    /* 网段名称，只允许输入中文、数字、大小写字母、英文下划线“_”及中划线“-”，不允许为空且不超过32字符。 (Optional) */
    SegmentName string `json:"segmentName"`

    /* 网段描述，允许输入UTF-8编码下的全部字符，不超过256字符。 (Optional) */
    Description string `json:"description"`

    /* 网络地址段 (Optional) */
    AddressPrefix string `json:"addressPrefix"`

    /* 网段可用ip数量 (Optional) */
    AvailableIpCount int `json:"availableIpCount"`

    /* IP服务商，取值为bgp或no_bgp，cn-north-1：bgp；cn-south-1：[bgp，no_bgp]；cn-east-1：[bgp，no_bgp]；cn-east-2：bgp (Optional) */
    Provider string `json:"provider"`

    /* 网段的起始地址 (Optional) */
    StartIp string `json:"startIp"`

    /* 网段的结束地址 (Optional) */
    EndIp string `json:"endIp"`
}
