/*
 * server
 *
 * <br/>https://ncloud.apigw.ntruss.com/server/v2
 *
 * API version: 2018-10-18T06:16:13Z
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package server

type CreateBlockStorageInstanceRequest struct {

	// 블럭스토리지명
BlockStorageName *string `json:"blockStorageName,omitempty"`

	// 블럭스토리지사이즈
BlockStorageSize *int64 `json:"blockStorageSize"`

	// 블럭스토리지설명
BlockStorageDescription *string `json:"blockStorageDescription,omitempty"`

	// 서버인스턴스번호
ServerInstanceNo *string `json:"serverInstanceNo"`

	// 디스크상세유형코드
DiskDetailTypeCode *string `json:"diskDetailTypeCode,omitempty"`
}
