package ecs

//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
// Code generated by Alibaba Cloud SDK Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

// Snapshot is a nested struct in ecs response
type Snapshot struct {
	SnapshotId        string                  `json:"SnapshotId" xml:"SnapshotId"`
	SnapshotName      string                  `json:"SnapshotName" xml:"SnapshotName"`
	Progress          string                  `json:"Progress" xml:"Progress"`
	ProductCode       string                  `json:"ProductCode" xml:"ProductCode"`
	SourceDiskId      string                  `json:"SourceDiskId" xml:"SourceDiskId"`
	SourceDiskType    string                  `json:"SourceDiskType" xml:"SourceDiskType"`
	RetentionDays     int                     `json:"RetentionDays" xml:"RetentionDays"`
	Encrypted         bool                    `json:"Encrypted" xml:"Encrypted"`
	SourceDiskSize    string                  `json:"SourceDiskSize" xml:"SourceDiskSize"`
	Description       string                  `json:"Description" xml:"Description"`
	CreationTime      string                  `json:"CreationTime" xml:"CreationTime"`
	Status            string                  `json:"Status" xml:"Status"`
	Usage             string                  `json:"Usage" xml:"Usage"`
	SourceStorageType string                  `json:"SourceStorageType" xml:"SourceStorageType"`
	RemainTime        int                     `json:"RemainTime" xml:"RemainTime"`
	ResourceGroupId   string                  `json:"ResourceGroupId" xml:"ResourceGroupId"`
	KMSKeyId          string                  `json:"KMSKeyId" xml:"KMSKeyId"`
	Tags              TagsInDescribeSnapshots `json:"Tags" xml:"Tags"`
}
