package ecs

//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
// Code generated by Alibaba Cloud SDK Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

// ReservedInstance is a nested struct in ecs response
type ReservedInstance struct {
	ReservedInstanceId   string                                    `json:"ReservedInstanceId" xml:"ReservedInstanceId"`
	RegionId             string                                    `json:"RegionId" xml:"RegionId"`
	ZoneId               string                                    `json:"ZoneId" xml:"ZoneId"`
	ReservedInstanceName string                                    `json:"ReservedInstanceName" xml:"ReservedInstanceName"`
	Description          string                                    `json:"Description" xml:"Description"`
	InstanceType         string                                    `json:"InstanceType" xml:"InstanceType"`
	Scope                string                                    `json:"Scope" xml:"Scope"`
	OfferingType         string                                    `json:"OfferingType" xml:"OfferingType"`
	InstanceAmount       int                                       `json:"InstanceAmount" xml:"InstanceAmount"`
	Status               string                                    `json:"Status" xml:"Status"`
	CreationTime         string                                    `json:"CreationTime" xml:"CreationTime"`
	ExpiredTime          string                                    `json:"ExpiredTime" xml:"ExpiredTime"`
	StartTime            string                                    `json:"StartTime" xml:"StartTime"`
	ResourceGroupId      string                                    `json:"ResourceGroupId" xml:"ResourceGroupId"`
	OperationLocks       OperationLocksInDescribeReservedInstances `json:"OperationLocks" xml:"OperationLocks"`
}
