/*
 * server
 *
 * <br/>https://ncloud.apigw.ntruss.com/server/v2
 *
 * API version: 2018-10-18T06:16:13Z
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package server

type PrivateSubnetInstance struct {

	// Private Subnet인스턴스번호
PrivateSubnetInstanceNo *string `json:"privateSubnetInstanceNo,omitempty"`

	// 서브넷
Subnet *string `json:"subnet,omitempty"`

	// Private Subnet설명
PrivateSubnetDescription *string `json:"privateSubnetDescription,omitempty"`

	// 생성일시
CreateDate *string `json:"createDate,omitempty"`

	// Private Subnet인스턴스상태
PrivateSubnetInstanceStatus *CommonCode `json:"privateSubnetInstanceStatus,omitempty"`

	// Private Subnet OP
PrivateSubnetInstanceOperation *CommonCode `json:"privateSubnetInstanceOperation,omitempty"`

	// Private Subnet상태이름
PrivateSubnetInstanceStatusName *string `json:"privateSubnetInstanceStatusName,omitempty"`

	// 리전
Region *Region `json:"region,omitempty"`

	// ZONE
Zone *Zone `json:"zone,omitempty"`
}
