// Package repl provides the structs and functions necessary to run REPL for
// HCL2. The REPL allows experimentation of HCL2 interpolations without having
// to run a HCL2 configuration.
package repl
