// Copyright 2018 JDCLOUD.COM
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// NOTE: This class is auto generated by the jdcloud code generator program.

package apis

import (
    "github.com/jdcloud-api/jdcloud-sdk-go/core"
    vpc "github.com/jdcloud-api/jdcloud-sdk-go/services/vpc/models"
)

type DescribeVpcPeeringRequest struct {

    core.JDCloudRequest

    /* Region ID  */
    RegionId string `json:"regionId"`

    /* vpcPeeringId ID  */
    VpcPeeringId string `json:"vpcPeeringId"`
}

/*
 * param regionId: Region ID (Required)
 * param vpcPeeringId: vpcPeeringId ID (Required)
 *
 * @Deprecated, not compatible when mandatory parameters changed
 */
func NewDescribeVpcPeeringRequest(
    regionId string,
    vpcPeeringId string,
) *DescribeVpcPeeringRequest {

	return &DescribeVpcPeeringRequest{
        JDCloudRequest: core.JDCloudRequest{
			URL:     "/regions/{regionId}/vpcPeerings/{vpcPeeringId}",
			Method:  "GET",
			Header:  nil,
			Version: "v1",
		},
        RegionId: regionId,
        VpcPeeringId: vpcPeeringId,
	}
}

/*
 * param regionId: Region ID (Required)
 * param vpcPeeringId: vpcPeeringId ID (Required)
 */
func NewDescribeVpcPeeringRequestWithAllParams(
    regionId string,
    vpcPeeringId string,
) *DescribeVpcPeeringRequest {

    return &DescribeVpcPeeringRequest{
        JDCloudRequest: core.JDCloudRequest{
            URL:     "/regions/{regionId}/vpcPeerings/{vpcPeeringId}",
            Method:  "GET",
            Header:  nil,
            Version: "v1",
        },
        RegionId: regionId,
        VpcPeeringId: vpcPeeringId,
    }
}

/* This constructor has better compatible ability when API parameters changed */
func NewDescribeVpcPeeringRequestWithoutParam() *DescribeVpcPeeringRequest {

    return &DescribeVpcPeeringRequest{
            JDCloudRequest: core.JDCloudRequest{
            URL:     "/regions/{regionId}/vpcPeerings/{vpcPeeringId}",
            Method:  "GET",
            Header:  nil,
            Version: "v1",
        },
    }
}

/* param regionId: Region ID(Required) */
func (r *DescribeVpcPeeringRequest) SetRegionId(regionId string) {
    r.RegionId = regionId
}

/* param vpcPeeringId: vpcPeeringId ID(Required) */
func (r *DescribeVpcPeeringRequest) SetVpcPeeringId(vpcPeeringId string) {
    r.VpcPeeringId = vpcPeeringId
}

// GetRegionId returns path parameter 'regionId' if exist,
// otherwise return empty string
func (r DescribeVpcPeeringRequest) GetRegionId() string {
    return r.RegionId
}

type DescribeVpcPeeringResponse struct {
    RequestID string `json:"requestId"`
    Error core.ErrorResponse `json:"error"`
    Result DescribeVpcPeeringResult `json:"result"`
}

type DescribeVpcPeeringResult struct {
    VpcPeering vpc.VpcPeering `json:"vpcPeering"`
}