package ecs

//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
// Code generated by Alibaba Cloud SDK Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

// NetworkInterfaceSet is a nested struct in ecs response
type NetworkInterfaceSet struct {
	NetworkInterfaceId   string                                      `json:"NetworkInterfaceId" xml:"NetworkInterfaceId"`
	Status               string                                      `json:"Status" xml:"Status"`
	Type                 string                                      `json:"Type" xml:"Type"`
	VpcId                string                                      `json:"VpcId" xml:"VpcId"`
	VSwitchId            string                                      `json:"VSwitchId" xml:"VSwitchId"`
	ZoneId               string                                      `json:"ZoneId" xml:"ZoneId"`
	PrivateIpAddress     string                                      `json:"PrivateIpAddress" xml:"PrivateIpAddress"`
	MacAddress           string                                      `json:"MacAddress" xml:"MacAddress"`
	NetworkInterfaceName string                                      `json:"NetworkInterfaceName" xml:"NetworkInterfaceName"`
	Description          string                                      `json:"Description" xml:"Description"`
	InstanceId           string                                      `json:"InstanceId" xml:"InstanceId"`
	CreationTime         string                                      `json:"CreationTime" xml:"CreationTime"`
	ResourceGroupId      string                                      `json:"ResourceGroupId" xml:"ResourceGroupId"`
	ServiceID            int                                         `json:"ServiceID" xml:"ServiceID"`
	ServiceManaged       bool                                        `json:"ServiceManaged" xml:"ServiceManaged"`
	SecurityGroupIds     SecurityGroupIdsInDescribeNetworkInterfaces `json:"SecurityGroupIds" xml:"SecurityGroupIds"`
	AssociatedPublicIp   AssociatedPublicIp                          `json:"AssociatedPublicIp" xml:"AssociatedPublicIp"`
	PrivateIpSets        PrivateIpSets                               `json:"PrivateIpSets" xml:"PrivateIpSets"`
	Ipv6Sets             Ipv6Sets                                    `json:"Ipv6Sets" xml:"Ipv6Sets"`
	Tags                 TagsInDescribeNetworkInterfaces             `json:"Tags" xml:"Tags"`
}
