package ram

//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
// Code generated by Alibaba Cloud SDK Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

// Role is a nested struct in ram response
type Role struct {
	RoleId                   string `json:"RoleId" xml:"RoleId"`
	CreateDate               string `json:"CreateDate" xml:"CreateDate"`
	AttachDate               string `json:"AttachDate" xml:"AttachDate"`
	Arn                      string `json:"Arn" xml:"Arn"`
	UpdateDate               string `json:"UpdateDate" xml:"UpdateDate"`
	Description              string `json:"Description" xml:"Description"`
	RoleName                 string `json:"RoleName" xml:"RoleName"`
	AssumeRolePolicyDocument string `json:"AssumeRolePolicyDocument" xml:"AssumeRolePolicyDocument"`
}
