package ecs

//Licensed under the Apache License, Version 2.0 (the "License");
//you may not use this file except in compliance with the License.
//You may obtain a copy of the License at
//
//http://www.apache.org/licenses/LICENSE-2.0
//
//Unless required by applicable law or agreed to in writing, software
//distributed under the License is distributed on an "AS IS" BASIS,
//WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//See the License for the specific language governing permissions and
//limitations under the License.
//
// Code generated by Alibaba Cloud SDK Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

// AutoSnapshotPolicy is a nested struct in ecs response
type AutoSnapshotPolicy struct {
	AutoSnapshotPolicyId   string `json:"AutoSnapshotPolicyId" xml:"AutoSnapshotPolicyId"`
	RegionId               string `json:"RegionId" xml:"RegionId"`
	AutoSnapshotPolicyName string `json:"AutoSnapshotPolicyName" xml:"AutoSnapshotPolicyName"`
	TimePoints             string `json:"TimePoints" xml:"TimePoints"`
	RepeatWeekdays         string `json:"RepeatWeekdays" xml:"RepeatWeekdays"`
	RetentionDays          int    `json:"RetentionDays" xml:"RetentionDays"`
	DiskNums               int    `json:"DiskNums" xml:"DiskNums"`
	VolumeNums             int    `json:"VolumeNums" xml:"VolumeNums"`
	CreationTime           string `json:"CreationTime" xml:"CreationTime"`
	Status                 string `json:"Status" xml:"Status"`
}
